summaryrefslogtreecommitdiffstats
path: root/common/t-b64.c
diff options
context:
space:
mode:
authorDaniel Baumann <daniel.baumann@progress-linux.org>2024-05-18 21:21:03 +0000
committerDaniel Baumann <daniel.baumann@progress-linux.org>2024-05-18 21:21:03 +0000
commit3675f65888fde5fddde20ff811638a338bf08ca6 (patch)
tree065688bbb6355a8a4784ec9c8e112cec342eac5e /common/t-b64.c
parentAdding upstream version 2.2.40. (diff)
downloadgnupg2-3675f65888fde5fddde20ff811638a338bf08ca6.tar.xz
gnupg2-3675f65888fde5fddde20ff811638a338bf08ca6.zip
Adding upstream version 2.2.43.upstream/2.2.43upstream
Signed-off-by: Daniel Baumann <daniel.baumann@progress-linux.org>
Diffstat (limited to 'common/t-b64.c')
-rw-r--r--common/t-b64.c134
1 files changed, 118 insertions, 16 deletions
diff --git a/common/t-b64.c b/common/t-b64.c
index 3b63872..16c079d 100644
--- a/common/t-b64.c
+++ b/common/t-b64.c
@@ -1,30 +1,24 @@
/* t-b64.c - Module tests for b64enc.c and b64dec.c
- * Copyright (C) 2008 Free Software Foundation, Inc.
+ * Copyright (C) 2008 Free Software Foundation, Inc.
+ * Copyright (C) 2008, 2023 g10 Code GmbH
*
* This file is part of GnuPG.
*
* GnuPG is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* GnuPG is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
*/
-/*
-
- As of now this is only a test program for manual tests.
-
- */
-
-
-
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
@@ -36,10 +30,112 @@
__FILE__,__LINE__, (a)); \
errcount++; \
} while(0)
+#define oops() do { fprintf (stderr, "%s:%d: ooops\n", \
+ __FILE__,__LINE__); \
+ exit (2); \
+ } while(0)
static int verbose;
static int errcount;
+
+/* Convert STRING consisting of hex characters into its binary
+ * representation and return it as an allocated buffer. The valid
+ * length of the buffer is returned at R_LENGTH. The string is
+ * delimited by end of string. The function returns NULL on
+ * error. */
+static void *
+hex2buffer (const char *string, size_t *r_length)
+{
+ const char *s;
+ unsigned char *buffer;
+ size_t length;
+
+ buffer = xmalloc (strlen(string)/2+1);
+ length = 0;
+ for (s=string; *s; s +=2 )
+ {
+ if (!hexdigitp (s) || !hexdigitp (s+1))
+ return NULL; /* Invalid hex digits. */
+ ((unsigned char*)buffer)[length++] = xtoi_2 (s);
+ }
+ *r_length = length;
+ return buffer;
+}
+
+
+static void
+test_b64decode (void)
+{
+ static struct {
+ const char *string; /* String to test. */
+ const char *title; /* title parameter. */
+ gpg_error_t err; /* expected error. */
+ const char *datastr; /* Expected data (hex encoded) */
+ } tests[] = {
+ { "YQ==", NULL, 0,
+ "61" },
+ { "YWE==", NULL, 0,
+ "6161" },
+ { "YWFh", NULL, 0,
+ "616161" },
+ { "YWFhYQ==", NULL, 0,
+ "61616161" },
+ { "YWJjZA==", NULL, 0,
+ "61626364" },
+ { "AA=", NULL, 0,
+ "00" },
+ { "AAEA=", NULL, 0,
+ "000100" },
+ { "/w==", NULL, 0,
+ "ff" },
+ { "oRQwEqADCgEDoQsGCSqGSIL3EgECAg==", NULL, 0,
+ "a1143012a0030a0103a10b06092a864882f712010202" },
+ { "oRQwEqADCgEDoQsGCSqGSIL3EgECA-==", NULL, GPG_ERR_BAD_DATA,
+ "a1143012a0030a0103a10b06092a864882f712010202" },
+ { "oRQwEqADCgEDoQsGCSqGSIL3EgECAg==", "", 0,
+ "" },
+ { "-----BEGIN PGP\n\n"
+ "oRQwEqADCgEDoQsGCSqGSIL3EgECAg==\n"
+ "-----END PGP\n", "", 0,
+ "a1143012a0030a0103a10b06092a864882f712010202" },
+
+ { "", NULL, 0,
+ "" }
+ };
+ int tidx;
+ gpg_error_t err;
+ void *data = NULL;
+ size_t datalen;
+ char *wantdata = NULL;
+ size_t wantdatalen;
+
+ for (tidx = 0; tidx < DIM(tests); tidx++)
+ {
+ xfree (wantdata);
+ if (!(wantdata = hex2buffer (tests[tidx].datastr, &wantdatalen)))
+ oops ();
+ xfree (data);
+ err = b64decode (tests[tidx].string, tests[tidx].title, &data, &datalen);
+ if (verbose)
+ fprintf (stderr, "%s:%d: test %d, err=%d, datalen=%zu\n",
+ __FILE__, __LINE__, tidx, err, datalen);
+ if (gpg_err_code (err) != tests[tidx].err)
+ fail (tidx);
+ else if (err)
+ pass ();
+ else if (wantdatalen != datalen)
+ fail (tidx);
+ else if (memcmp (wantdata, data, datalen))
+ fail (tidx);
+ else
+ pass ();
+ }
+ xfree (wantdata);
+ xfree (data);
+}
+
+
static void
test_b64enc_pgp (const char *string)
{
@@ -101,6 +197,7 @@ test_b64enc_file (const char *fname)
pass ();
}
+
static void
test_b64dec_file (const char *fname)
{
@@ -150,6 +247,7 @@ main (int argc, char **argv)
{
int do_encode = 0;
int do_decode = 0;
+ int do_pgpdecode = 0;
if (argc)
{ argc--; argv++; }
@@ -169,13 +267,17 @@ main (int argc, char **argv)
do_decode = 1;
argc--; argv++;
}
+ else if (argc)
+ do_pgpdecode = 1;
if (do_encode)
test_b64enc_file (argc? *argv: NULL);
else if (do_decode)
test_b64dec_file (argc? *argv: NULL);
- else
+ else if (do_pgpdecode)
test_b64enc_pgp (argc? *argv: NULL);
+ else
+ test_b64decode ();
return !!errcount;
}
@@ -1,30 +1,24 @@
* This file is part of GnuPG.
*
- * GnuPG is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 3 of the License, or
- * (at your option) any later version.
+ * This file is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
*
@@ -1,30 +1,24 @@
*
- * GnuPG is distributed in the hope that it will be useful,
+ * This file is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the